from dolfin import *

import matplotlib.pyplot as plt
import matplotlib.tri as tri

import numpy as np
from mshr import *


T = 64              # final time
num_steps =  640    # number of time steps
dt = T / num_steps # time step size
r = 2.5

# Create mesh and define function space

N = 32
domain = Rectangle(dolfin.Point(0.0,0.0),dolfin.Point(pi,pi))
mesh = generate_mesh(domain,N)

X = FiniteElement("CG",mesh.ufl_cell(),1)
XX = X * X
V = FunctionSpace(mesh, XX)

# Define exact solutions

Amu0 = Expression((('0.0','(6/(pow(a,3)))*(t*(t - a))*(rho*(exp(-4*t) - exp(-16*t))*sin(2*x[0])*sin(2*x[1]) + (2*exp(-16*t) - (1/2)*exp(-4*t))*sin(2*x[0])*sin(2*x[1]))')), a = T, rho = r, t = dt, degree=4)
Amu1 = Expression((('0.0','(6/(pow(a,3)))*(t*(t - a))*(16*exp(-16*t) - 4*exp(-4*t))*sin(2*x[0])*sin(2*x[1]) + (6/(pow(a,3)))*(2*t - a)*(exp(-4*t) - exp(-16*t))*sin(2*x[0])*sin(2*x[1])')), a = T, t = dt, degree=4)
EIvw = Expression((('0.0','(3/2)*sin(2*x[0])*sin(2*x[1])')), degree=4)


# Define boundary condition

def boundary(x, on_boundary):
    return on_boundary

noslip = Constant(0)
bc = DirichletBC(V.sub(0),noslip,boundary)

# Interpolate initial value

Evw = interpolate(EIvw, V)

# Define variational problems

# Solve for mu1 prime
mu1 = TrialFunction(V)
vh = TestFunction(V)

a1 = inner(grad(mu1), grad(vh))*dx
L1 = dot(Amu1,vh)*dx

mu1 = Function(V)
solve(a1 == L1, mu1, bc)

# Construct the null-controller UN

AUN = interpolate(Amu0, V)
UN = Function(V)
UN = mu1 + AUN

# Original problem Variational Formulation

v = TrialFunction(V)
w = TestFunction(V)

a = dot(v,w)*dx - dt*dot(grad(v[1]), grad(w[0]))*dx + dt*(dot(grad(v[0]), grad(w[1])) + r*dot(grad(v[1]), grad(w[1])))*dx
L = dot(w,Evw)*dx + dt*UN[1]*w[1]*dx

u = Function(V)
solve(a == L, u, bc)

print('Velocity L2 norm:',sqrt(assemble(dot(u,u)*dx)))

# Time-stepping
t = dt
for n in range(num_steps):

    # Update current time
    t += dt
    Amu0.t = t
    Amu1.t = t
    
    #u_n = interpolate(Amu1, V)
    AUN = interpolate(Amu0, V)
    Evw.assign(u)

    # Compute solution
    solve(a1 == L1, mu1, bc)
    UN = mu1 + AUN
    solve(a == L, u, bc)
    print('Velocity L2 norm:',sqrt(assemble(dot(u,u)*dx)))
